@@ -1,8 +1,8 @@
 /***************************************************************************
-    date                 : Feb 05 2006
-    version              : 0.31
-    copyright            : (C) 2004-2006 by Holger Danielsson
-     email                : holger.danielsson@t-online.de
+    date                 : Mar 21 2007
+    version              : 0.40
+    copyright            : (C) 2004-2007 by Holger Danielsson
+    email                : holger.danielsson@versanet.de
 ***************************************************************************/
 
 /***************************************************************************
@@ -13,13 +13,15 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
- 
+
+#include "codecompletion.h"
+
 #include <qregexp.h>
 #include <qfile.h>
 #include <qtimer.h>
 #include <qdict.h>
 
-#include <kdebug.h>
+#include "kiledebug.h"
 #include <klocale.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
@@ -29,24 +31,20 @@
 
 #include "kileinfo.h"
 #include "kileviewmanager.h"
-#include "codecompletion.h"
 #include "kileconfig.h"
 #include "kileedit.h"
+#include "kiledebug.h"
 
 namespace KileDocument
 {
 
-	//static QRegExp::QRegExp reRef("^\\\\(pageref|ref)\\{");
-	//static QRegExp::QRegExp reCite("^\\\\(c|C|noc)(ite|itep|itet|itealt|itealp|iteauthor|iteyear|iteyearpar|itetext)\\{");
-	//static QRegExp::QRegExp reRefExt("^\\\\(pageref|ref)\\{[^\\{\\}\\\\]+,$");
-	//static QRegExp::QRegExp reCiteExt("^\\\\(c|C|noc)(ite|itep|itet|itealt|itealp|iteauthor|iteyear|iteyearpar|itetext)\\{[^\\{\\}\\\\]+,$");
-	
 	static QRegExp::QRegExp reRef;
 	static QRegExp::QRegExp reRefExt;
 	static QRegExp::QRegExp reCite;
 	static QRegExp::QRegExp reCiteExt;
-	static QRegExp::QRegExp reNotRefChars("[^a-zA-z0-9_@\\.\\+\\-\\*\\:]");
-	
+	static QRegExp::QRegExp reNotRefChars("[^a-zA-Z0-9_@\\.\\+\\-\\*\\:]");
+	static QRegExp::QRegExp reNotCiteChars("[^a-zA-Z0-9_@\\-\\:]");
+
 	CodeCompletion::CodeCompletion(KileInfo *info) : m_ki(info), m_view(0L)
 	{
 		m_firstconfig = true;
@@ -54,6 +52,11 @@ namespace KileDocument
 		m_undo = false;
 		m_ref = false;
 		m_kilecompletion = false;
+		m_keylistType = CodeCompletion::ctNone;
+
+		// local abbreviation list
+		m_abbrevListview = 0L;
+		m_localAbbrevFile = locateLocal("appdata", "complete/abbreviation/", true) + "kile-abbrevs.cwl";
 
 		//reRef.setPattern("^\\\\(pageref|ref|xyz)\\{");
 		m_completeTimer = new QTimer( this );
@@ -97,22 +100,72 @@ namespace KileDocument
 		return m_mode;
 	}
 
+	CodeCompletion::Type CodeCompletion::insideReference(QString &startpattern)
+	{
+		if ( m_view->getDoc() )
+		{
+			uint column = m_view->cursorColumnReal();
+			QString currentline = m_view->getDoc()->textLine(m_view->cursorLine()).left(column);
+			int pos = currentline.findRev('\\');
+			if ( pos >= 0 )
+			{
+				QString command = currentline.mid(pos,column-pos);
+				KILE_DEBUG() << "pos=" << pos << ",column=" << column << ",currentline=" << currentline << ",command=" << command << endl;
+
+				if( command.find(reRefExt) != -1 ){
+					KILE_DEBUG() << "reRefExt" << endl;
+					startpattern = command.right(command.length()-reRefExt.cap(0).length());
+					KILE_DEBUG() << "startpattern=" << startpattern << endl;
+					if ( startpattern.find(reNotRefChars) == -1 ){
+						return CodeCompletion::ctReference ;
+					}
+				}
+ 				else if ( command.find(reRef) != -1 ){
+					startpattern = command.right(command.length()-reRef.cap(0).length());
+					KILE_DEBUG() << "startpattern=" << startpattern << endl;
+					if ( startpattern.find(reNotRefChars) == -1 ){
+						return CodeCompletion::ctReference ;
+					}
+				}
+				else if( command.find(reCiteExt) != -1 ){
+					KILE_DEBUG() << "reCiteExt" << endl;
+					startpattern = command.right(command.length()-reCiteExt.cap(0).length());
+					KILE_DEBUG() << "startpattern=" << startpattern << endl;
+					if ( startpattern.find(reNotCiteChars) == -1 ){
+						return CodeCompletion::ctCitation;
+					}
+				}
+				else if ( command.find(reCite) != -1 ){
+					startpattern = command.right(command.length()-reCite.cap(0).length());
+					KILE_DEBUG() << "startpattern=" << startpattern << endl;
+					if ( startpattern.find(reNotCiteChars) == -1 ){
+						return CodeCompletion::ctCitation;
+					}
+				}
+			}
+		}
+		return CodeCompletion::ctNone;
+	}
+
 	//////////////////// configuration ////////////////////
 
 	void CodeCompletion::readConfig(KConfig *config)
 	{
-		kdDebug() << "=== CodeCompletion::readConfig ===================" << endl;
+		KILE_DEBUG() << "=== CodeCompletion::readConfig ===================" << endl;
 
 		// save normal parameter
-		//kdDebug() << "   read bool entries" << endl;
+		//KILE_DEBUG() << "   read bool entries" << endl;
 		m_isenabled = KileConfig::completeEnabled();
 		m_setcursor = KileConfig::completeCursor();
 		m_setbullets = KileConfig::completeBullets();
 		m_closeenv = KileConfig::completeCloseEnv();
 		m_autocomplete = KileConfig::completeAuto();
 		m_autocompletetext = KileConfig::completeAutoText();
+		m_autocompleteabbrev = KileConfig::completeAutoAbbrev();
 		m_latexthreshold = KileConfig::completeAutoThreshold();
 		m_textthreshold = KileConfig::completeAutoTextThreshold();
+		m_citationMove = KileConfig::completeCitationMove();
+		m_autoDollar = KileConfig::autoInsertDollar();
 
 		// we need to read some of Kate's config flags
 		readKateConfigFlags(config);
@@ -121,14 +174,14 @@ namespace KileDocument
 		// and when the list of files changes
 		if ( m_firstconfig || KileConfig::completeChangedLists()  || KileConfig::completeChangedCommands() )
 		{
-			kdDebug() << "   set regexp for references..." << endl;
+			KILE_DEBUG() << "   set regexp for references..." << endl;
 			setReferences();
-			
-			kdDebug() << "   read wordlists..." << endl;
+
+			KILE_DEBUG() << "   read wordlists..." << endl;
 			// wordlists for Tex/Latex mode
 			QStringList files = KileConfig::completeTex();
 			setWordlist( files, "tex", &m_texlist );
-		
+
 			// wordlist for dictionary mode
 			files = KileConfig::completeDict();
 			setWordlist( files, "dictionary", &m_dictlist );
@@ -150,22 +203,33 @@ namespace KileDocument
 		m_autobrackets = ( config->readNumEntry("Basic Config Flags",0) & cfAutoBrackets );
 		m_autoindent   = ( config->readNumEntry("Indentation Mode",0) > 0 );
 	}
-	
+
+	// save local abbreviation changes
+	// (for example before a new configuration should be read)
+
+	void CodeCompletion::saveLocalChanges()
+	{
+		KILE_DEBUG() << "=== CodeCompletion::saveLocalChanges ===================" << endl;
+		m_abbrevListview->saveLocalAbbreviation(m_localAbbrevFile);
+	}
+
 	//////////////////// references and citations ////////////////////
-	
+
 	void CodeCompletion::setReferences()
 	{
 		// build list of references
 		QString references = getCommandList(KileDocument::CmdAttrReference);
-		reRef.setPattern("^\\\\(pageref|ref|fref|Fref|eqref" + references + ")\\{");
-		reRefExt.setPattern("^\\\\(pageref|ref|fref|Fref|eqref" + references + ")\\{[^\\{\\}\\\\]+,$");
-		
+		references.replace("*","\\*");
+		reRef.setPattern("^\\\\(" + references + ")\\{");
+		reRefExt.setPattern("^\\\\(" + references + ")\\{[^\\{\\}\\\\]+,");
+
 		// build list of citations
 		QString citations = getCommandList(KileDocument::CmdAttrCitations);
+		citations.replace("*","\\*");
 		reCite.setPattern("^\\\\(((c|C|noc)(ite|itep|itet|itealt|itealp|iteauthor|iteyear|iteyearpar|itetext))" + citations +  ")\\{");
-		reCiteExt.setPattern("^\\\\(((c|C|noc)(ite|itep|itet|itealt|itealp|iteauthor|iteyear|iteyearpar|itetext))" + citations + ")\\{[^\\{\\}\\\\]+,$");
+		reCiteExt.setPattern("^\\\\(((c|C|noc)(ite|itep|itet|itealt|itealp|iteauthor|iteyear|iteyearpar|itetext))" + citations + ")\\{[^\\{\\}\\\\]+,");
 	}
-		
+
 	QString CodeCompletion::getCommandList(KileDocument::CmdAttribute attrtype)
 	{
 		QStringList cmdlist;
@@ -173,24 +237,26 @@ namespace KileDocument
 
 		// get info about user defined references
 		KileDocument::LatexCommands *cmd = m_ki->latexCommands();
-		cmd->commandList(cmdlist,attrtype,true);
-	
+		cmd->commandList(cmdlist,attrtype,false);
+
 		// build list of references
 		QString commands = QString::null;
-		for ( it=cmdlist.begin(); it != cmdlist.end(); ++it ) 
+		for ( it=cmdlist.begin(); it != cmdlist.end(); ++it )
 		{
-			commands += "|" + (*it).mid(1);
+			if ( cmd->isStarredEnv(*it) )
+				commands += '|' + (*it).mid(1) + '*';
+			commands += '|' + (*it).mid(1);
 		}
 		return commands;
 	}
-	
+
 	//////////////////// wordlists ////////////////////
-	
+
 	void CodeCompletion::setWordlist( const QStringList &files, const QString &dir,
 	                                  QValueList<KTextEditor::CompletionEntry> *entrylist
 	                                )
 	{
-		
+
 		// read wordlists from files
 		QStringList wordlist;
 		for ( uint i = 0; i < files.count(); ++i )
@@ -198,7 +264,7 @@ namespace KileDocument
 			// if checked, the wordlist has to be read
 			if ( files[ i ].at( 0 ) == '1' )
 			{
-				readWordlist( wordlist, dir + "/" + files[ i ].right( files[ i ].length() - 2 ) + ".cwl" );
+				readWordlist( wordlist, dir + '/' + files[i].right( files[i].length()-2 ) + ".cwl", true );
 			}
 		}
 
@@ -208,11 +274,28 @@ namespace KileDocument
 			addCommandsToTexlist(wordlist);
 			setCompletionEntriesTexmode( entrylist, wordlist );
 		}
-		else
+		else if ( dir == "dictionary" )
 		{
 			wordlist.sort();
 			setCompletionEntries( entrylist, wordlist );
 		}
+		else if ( dir == "abbreviation" )
+		{
+			// read local wordlist
+			QStringList localWordlist;
+			readWordlist(localWordlist, QString::null, false);
+
+			// add local/global wordlists to the abbreviation view
+			m_abbrevListview->init(&wordlist,&localWordlist);
+
+			// finally add local wordlists to the abbreviation list
+			QStringList::ConstIterator it;
+			for ( it=localWordlist.begin(); it!=localWordlist.end(); ++it )
+				wordlist.append( *it );
+
+			wordlist.sort();
+			setCompletionEntries( entrylist, wordlist );
+		}
 	}
 
 	void CodeCompletion::addCommandsToTexlist(QStringList &wordlist)
@@ -224,11 +307,11 @@ namespace KileDocument
 		// get info about user defined commands and environments
 		KileDocument::LatexCommands *cmd = m_ki->latexCommands();
 		cmd->commandList(cmdlist,KileDocument::CmdAttrNone,true);
-	
+
 		// add entries to wordlist
-		for ( it=cmdlist.begin(); it != cmdlist.end(); ++it ) 
+		for ( it=cmdlist.begin(); it != cmdlist.end(); ++it )
 		{
-			if ( cmd->commandAttributes(*it,attr) ) 
+			if ( cmd->commandAttributes(*it,attr) )
 			{
 				QString command,eos;
 				QStringList entrylist;
@@ -242,28 +325,28 @@ namespace KileDocument
 					command = (*it);
 					// eos = QString::null;
 				}
-				
+
 				// get all possibilities into a stringlist
 				entrylist.append( command + eos );
 				if ( ! attr.option.isEmpty() )
 					entrylist.append( command + eos + "[option]" );
 				if ( attr.starred )
 				{
-					entrylist.append( command + "*" + eos );
+					entrylist.append( command + '*' + eos );
 					if ( ! attr.option.isEmpty() )
-						entrylist.append( command + "*" + eos + "[option]" );
+						entrylist.append( command + '*' + eos + "[option]" );
 				}
 
 				// finally append entries to wordlist
 				QStringList::ConstIterator itentry;
-				for ( itentry=entrylist.begin(); itentry != entrylist.end(); ++itentry ) 
+				for ( itentry=entrylist.begin(); itentry != entrylist.end(); ++itentry )
 				{
 					QString entry = (*itentry);
 					if ( ! attr.parameter.isEmpty()  )
 						entry += "{param}";
 					if ( attr.type == KileDocument::CmdAttrList )
 						entry += "\\item";
-					wordlist.append( entry ); 
+					wordlist.append( entry );
 				}
 			}
 		}
@@ -273,10 +356,10 @@ namespace KileDocument
 
 	void CodeCompletion::completeWord(const QString &text, CodeCompletion::Mode mode)
 	{
-		kdDebug() << "==CodeCompletion::completeWord(" << text << ")=========" << endl;
-		//kdDebug() << "\tm_view = " << m_view << endl;
+		KILE_DEBUG() << "==CodeCompletion::completeWord(" << text << ")=========" << endl;
+		KILE_DEBUG() << "\tm_view = " << m_view << endl;
 		if ( !m_view) return;
-		//kdDebug() << "ok" << endl;
+		KILE_DEBUG() << "ok" << endl;
 
 		// remember all parameters (view, pattern, length of pattern, mode)
 		m_text = text;
@@ -295,7 +378,7 @@ namespace KileDocument
 			QString s = doc->textLine(m_ycursor);
 			int pos = s.findRev("\\",m_xcursor);
 			if (pos < 0) {
-				//kdDebug() << "\tfound no backslash! s=" << s << endl;
+				KILE_DEBUG() << "\tfound no backslash! s=" << s << endl;
 				return;
 			}
 			m_xstart = pos;
@@ -308,14 +391,10 @@ namespace KileDocument
 		QValueList<KTextEditor::CompletionEntry> list;
 		switch ( m_mode )
 		{
-				case cmLatex:
-        //kdDebug() << "mode = cmLatex" << endl;
-        list = m_texlist;
-        appendNewCommands(list);
-        break;
+				case cmLatex: // fall through here
 				case cmEnvironment:
-        //kdDebug() << "mode = cmEnvironment" << endl;
-				list = m_texlist;				
+				list = m_texlist;
+				appendNewCommands(list);
 				break;
 				case cmDictionary:
 				list = m_dictlist;
@@ -335,8 +414,8 @@ namespace KileDocument
 		QString entry, type;
 		QString pattern = ( m_mode != cmEnvironment ) ? text : "\\begin{" + text;
 		uint n = countEntries( pattern, &list, &entry, &type );
-		
-		//kdDebug() << "entries = " << n << endl;
+
+		KILE_DEBUG() << "entries = " << n << endl;
 
 		// nothing to do
 		if ( n == 0 )
@@ -385,21 +464,24 @@ namespace KileDocument
 		}
 	}
 
-	void CodeCompletion::completeFromList(const QStringList *list )
+	void CodeCompletion::completeFromList(const QStringList *list,const QString &pattern)
 	{
 		KTextEditor::CompletionEntry e;
 
-		//kdDebug() << "completeFromList: " << list->count() << " items" << endl;
+		KILE_DEBUG() << "completeFromList: " << list->count() << " items" <<   " << pattern=" << pattern<< endl;
+		QStringList sortedlist( *list );
+		sortedlist.sort();
+
 		m_labellist.clear();
 		QStringList::ConstIterator it;
-		QStringList::ConstIterator itend(list->end());
-		for ( it = list->begin(); it != itend; ++it )
+		QStringList::ConstIterator itend(sortedlist.end());
+		for ( it = sortedlist.begin(); it != itend; ++it )
 		{
 			e.text = *it;
 			m_labellist.append(  e );
 		}
 
-		completeWord("", cmLabel);
+		completeWord(pattern, cmLabel);
 	}
 
 	//////////////////// completion was done ////////////////////
@@ -422,6 +504,7 @@ namespace KileDocument
 
 	void CodeCompletion::CompletionAborted()
 	{
+		KILE_DEBUG() << "CodeCompletion::CompletionAborted()" << endl;
 		// aborted: undo if kile completion is active
 		if ( m_inprogress && m_undo && m_view )
 		{
@@ -450,12 +533,12 @@ namespace KileDocument
 	QString CodeCompletion::filterCompletionText( const QString &text, const QString &type )
 	{
 		static QRegExp::QRegExp reEnv = QRegExp("^\\\\(begin|end)[^a-zA-Z]+");
-		//kdDebug() << "   complete filter: " << text << " type " << type << endl;
+		KILE_DEBUG() << "   complete filter: " << text << " type " << type << endl;
 		m_type = getType( text );    // remember current type
-    
+
 		if ( text!="\\begin{}" && reEnv.search(text)!=-1 )
 			m_mode = cmEnvironment;
-    
+
 		// check the cursor position, because the user may have
 		// typed some characters or the backspace key. This also
 		// changes the length of the current pattern.
@@ -487,7 +570,7 @@ namespace KileDocument
 				prefix = QString::null;
 				if ( m_autoindent )
 				{
-					if ( col-m_textlen>0 ) 
+					if ( col-m_textlen>0 )
 					{
 						prefix = textline.left(col-m_textlen);
 						if ( prefix.right(7) == "\\begin{" )
@@ -501,14 +584,14 @@ namespace KileDocument
 				{
 					doc->removeText(row,col,row,col+1);
 				}
-				if ( m_xstart>=7 && textline.mid(m_xstart-7,7) == "\\begin{" ) 
+				if ( m_xstart>=7 && textline.mid(m_xstart-7,7) == "\\begin{" )
 				{
 					m_textlen += 7;
-				} 
-				else if ( m_xstart>=5 && textline.mid(m_xstart-5,5) == "\\end{" ) 
+				}
+				else if ( m_xstart>=5 && textline.mid(m_xstart-5,5) == "\\end{" )
 				{
 					m_textlen += 5;
-				} 
+				}
 				break;
 				case cmDictionary:
 				s = text;
@@ -518,13 +601,18 @@ namespace KileDocument
 				break;
 				case cmLabel:
 				s = buildLabelText( text );
+				if ( m_keylistType==CodeCompletion::ctReference
+				     || (m_keylistType==CodeCompletion::ctCitation && m_citationMove) )
+				{
+					m_xoffset = s.length() + 1;
+				}
 				break;
 				case cmDocumentWord:
 				s = text;
 				break;
         default : s = text; break;
 		}
-		
+
 		if ( s.length() > m_textlen )
 			return s.right( s.length() - m_textlen );
 		else
@@ -567,7 +655,7 @@ namespace KileDocument
 			s += s_bullet;
 
 		if ( m_closeenv && start != "end" )
-			s += "\n" + whitespace + "\\end{" + envname + "}\n";
+			s += '\n' + whitespace + "\\end{" + envname + "}\n";
 
 		// place cursor
 		if ( m_setcursor )
@@ -580,7 +668,10 @@ namespace KileDocument
 			else
 			{
 				ypos = 0;
-				xpos = 9 + envname.length();
+				if( parameter.left(2) == "[<" )
+					xpos = 10 + envname.length();
+				else
+					xpos = 9 + envname.length();
 			}
 		}
 
@@ -592,7 +683,7 @@ namespace KileDocument
 		QString whitespace = s;
 		for ( uint i=0; i<whitespace.length(); ++i )
 		{
-			if ( ! whitespace[i].isSpace() ) 
+			if ( ! whitespace[i].isSpace() )
 				whitespace[i] = ' ';
 		}
 		return whitespace;
@@ -656,22 +747,30 @@ namespace KileDocument
 		{
 			switch ( text[ i ] )
 			{
+					case '<':
 					case '{':
 					case '(':
-					case '[':                    // insert character
+					case '[':		// insert character
 					s += text[ i ];
 					if ( xpos == 0 )
 					{
 						// remember position after first brace
-						xpos = i + 1;
+						if(text[i] == '[' && (i+1) < text.length() &&  text[i+1] == '<'){
+							xpos = i + 2;
+							s += text[i+1];
+							i++;
+						}// special handling for '[<'
+						else
+							xpos = i + 1;
 						// insert bullet, if this is no cursorposition
-						if ( ( ! m_setcursor ) && m_setbullets )
+						if ( ( ! m_setcursor ) && m_setbullets && !( text[i] == '[' && (i+1) < text.length() &&  text[i+1] == '<' ))
 							s += s_bullet;
 					}
 					// insert bullets after following braces
-					else if ( m_setbullets )
+					else if ( m_setbullets && !( text[i] == '[' && (i+1) < text.length() &&  text[i+1] == '<' ) )
 						s += s_bullet;
 					break;
+					case '>':
 					case '}':
 					case ')':
 					case ']':                    // insert character
@@ -690,9 +789,9 @@ namespace KileDocument
 						foundgroup = true;
 						s.truncate( s.length() - 1 );
 						if ( m_setbullets )
-							s += " " + s_bullet + " ";
+							s += ' ' + s_bullet + ' ';
 						else
-							s += " ";
+							s += ' ';
 					}
 					else
 						s += text[ i ];
@@ -748,6 +847,7 @@ namespace KileDocument
 	}
 
 	// strip all names enclosed in braces
+	// consider also beamer like stuff [<...>] and <...>
 
 	QString CodeCompletion::stripParameter( const QString &text )
 	{
@@ -755,19 +855,22 @@ namespace KileDocument
 		const QChar *ch = text.unicode();
 		bool ignore = false;
 
+
 		for ( uint i = 0; i < text.length(); ++i )
 		{
 			switch ( *ch )
 			{
+					case '[':
 					case '{':
 					case '(':
-					case '[':
+					case '<':
 					s += *ch;
 					ignore = true;
 					break;
+					case ']':
 					case '}':
 					case ')':
-					case ']':
+					case '>':
 					s += *ch;
 					ignore = false;
 					break;
@@ -786,9 +889,11 @@ namespace KileDocument
 
 	//////////////////// read wordlists  ////////////////////
 
-	void CodeCompletion::readWordlist( QStringList &wordlist, const QString &filename )
+	void CodeCompletion::readWordlist( QStringList &wordlist, const QString &filename, bool global )
 	{
-		QString file = KGlobal::dirs() ->findResource( "appdata", "complete/" + filename );
+		QString file = ( global )
+		             ? KGlobal::dirs()->findResource( "appdata", "complete/" + filename )
+		             : m_localAbbrevFile;
 		if ( file.isEmpty() ) return;
 
 		QFile f( file );
@@ -815,14 +920,14 @@ namespace KileDocument
 
 		KTextEditor::CompletionEntry e;
 		QStringList::ConstIterator it;
-		
+
 		// build new entries
-		for ( it=wordlist.begin(); it != wordlist.end(); ++it ) 
+		for ( it=wordlist.begin(); it != wordlist.end(); ++it )
 		{
 			// set CompletionEntry
 			e.text = *it;
 			e.type = "";
-			
+
 			// add new entry
 			if ( list->findIndex(e) == -1 )
 				list->append(e);
@@ -839,11 +944,11 @@ namespace KileDocument
 		// order: \abc, \abc[], \abc{}, \abc*, \abc*[], \abc*{}, \abcd, \abcD
 		QStringList keylist;
 		QMap<QString,QString> map;
-		
+
 		for ( uint i=0; i< wordlist.count(); ++i )
 		{
 			QString s = wordlist[i];
-			for ( uint j=0; j<s.length(); ++j ) 
+			for ( uint j=0; j<s.length(); ++j )
 			{
 				QChar ch = s[j];
 				if ( ch>='A' && ch<='Z' )
@@ -868,18 +973,18 @@ namespace KileDocument
 				keylist.append(s);
 			}
 		}
-		
+
 		// sort mapped keys
 		keylist.sort();
-		
+
 		// build new entries: get the sorted keys and insert
-		// the real entries, which are saved in the map. 
-		// if the last 5 chars of an environment are '\item', it is a 
+		// the real entries, which are saved in the map.
+		// if the last 5 chars of an environment are '\item', it is a
 		// list environment, where the '\item' tag is also inserted
 		KTextEditor::CompletionEntry e;
 		QStringList::ConstIterator it;
-		
-		for ( it=keylist.begin(); it != keylist.end(); ++it ) 
+
+		for ( it=keylist.begin(); it != keylist.end(); ++it )
 		{
 			// get real entry
 			QString s = map[*it];
@@ -924,12 +1029,45 @@ namespace KileDocument
 		return n;
 	}
 
+	QString CodeCompletion::findExpansion(const QString &abbrev)
+	{
+		QValueList<KTextEditor::CompletionEntry>::Iterator it;
+
+		for ( it=m_abbrevlist.begin(); it!=m_abbrevlist.end(); ++it )
+		{
+			QString s = (*it).text;
+			int index = s.find("=");
+			if ( index>=0 && s.left(index)==abbrev )
+				 return s.right( s.length()-index-1 );
+		}
+
+		return QString::null;
+	}
+
 	void CodeCompletion::editComplete(Kate::View *view, Mode mode)
 	{
-		m_view = view;
+		KILE_DEBUG() << "void CodeCompletion::editComplete(Kate::View *view, Mode "<< mode << ")" << endl;
+		KILE_DEBUG() << "m_inprogress=" << m_inprogress << ", isActive()=" << isActive() << endl;
+		m_view = view; 
 
-		if ( !m_view || !isActive() || inProgress() )
+		if ( !m_view || !isActive() )
 			return ;
+		
+		m_inprogress=true;
+
+		KILE_DEBUG() << "proceeded" << endl;
+		// check for a special case: call from inside of a reference command
+		if ( mode==cmLatex )
+		{
+			QString startpattern;
+			CodeCompletion::Type reftype = insideReference(startpattern);
+			if ( reftype != CodeCompletion::ctNone )
+			{
+				m_ref = true;
+				editCompleteList(reftype,startpattern);
+				return;
+			}
+		}
 
 		QString word;
 		Type type;
@@ -940,42 +1078,36 @@ namespace KileDocument
 				mode = cmDictionary;
 			}
 
-			if ( type == ctNone )
+			if ( type == CodeCompletion::ctNone )
 				completeWord(word, mode);
 			else
 				editCompleteList(type);
 		}
-		//little hack to make multiple insertions like \cite{test1,test2} possible (only when
-		//completion is invoke explicitly using ctrl+space.
-		else if ( m_view->getDoc() )
-		{
-			QString currentline = m_view->getDoc()->textLine(m_view->cursorLine()).left(m_view->cursorColumnReal() + 1);
-			if ( currentline.find(reCiteExt) != -1 )
-				editCompleteList(ctCitation);
-			else if ( currentline.find(reRefExt) != -1 )
-				editCompleteList(ctReference);
-		}
 	}
 
-	void CodeCompletion::editCompleteList(Type type )
+	void CodeCompletion::editCompleteList(Type type,const QString &pattern)
 	{
-		//kdDebug() << "==editCompleteList=============" << endl;
+		KILE_DEBUG() << "==editCompleteList=============" << endl;
+		m_keylistType = type;
 		if ( type == ctReference )
-			completeFromList(info()->allLabels());
+			completeFromList(info()->allLabels(),pattern);
 		else if ( type == ctCitation )
-			completeFromList(info()->allBibItems());
-    else
-      kdWarning() << "unsupported type in CodeCompletion::editCompleteList" << endl;
+			completeFromList(info()->allBibItems(),pattern);
+		else
+		{
+			m_keylistType = CodeCompletion::ctNone;
+			kdWarning() << "unsupported type in CodeCompletion::editCompleteList" << endl;
+		}
 	}
 
 	//////////////////// slots for code completion ////////////////////
 
 	void CodeCompletion::slotCompletionDone(KTextEditor::CompletionEntry entry)
 	{
-		//kdDebug() << "==slotCompletionDone (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
+		KILE_DEBUG() << "==slotCompletionDone (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
 		CompletionDone(entry);
-
-		// if kile completion was active, look if we need to show an additonal list
+		
+		// if kile completion was active, look if we need to show an additional list
 		if ( m_kilecompletion )
 		{
 			m_kilecompletion = false;
@@ -985,6 +1117,7 @@ namespace KileDocument
 				if ( (m_type==CodeCompletion::ctReference && info()->allLabels()->count()>0)  ||
 					  (m_type==CodeCompletion::ctCitation  && info()->allBibItems()->count()>0) )
 				{
+					m_keylistType = m_type;
 					m_ref = true;
 		 			m_completeTimer->start(20,true);
 				}
@@ -994,25 +1127,25 @@ namespace KileDocument
 
 	void CodeCompletion::slotCompleteValueList()
 	{
-		//kdDebug() << "==slotCompleteValueList (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
+		KILE_DEBUG() << "==slotCompleteValueList (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
 		m_completeTimer->stop();
 		editCompleteList(getType());
 	}
 
 	void CodeCompletion::slotCompletionAborted()
 	{
-		//kdDebug() << "==slotCompletionAborted (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
+		KILE_DEBUG() << "==slotCompletionAborted (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
 		CompletionAborted();
 	}
 
 	void CodeCompletion::slotFilterCompletion( KTextEditor::CompletionEntry* c, QString *s )
 	{
-		//kdDebug() << "==slotFilterCompletion (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
+		KILE_DEBUG() << "==slotFilterCompletion (" << m_kilecompletion << "," << m_inprogress << ")=============" << endl;
 		if ( inProgress() )                 // dani 28.09.2004
 		{
-			//kdDebug() << "\tin progress: s=" << *s << endl;
+			KILE_DEBUG() << "\tin progress: s=" << *s << endl;
 			*s = filterCompletionText( c->text, c->type );
-			//kdDebug() << "\tfilter --->" << *s << endl;
+			KILE_DEBUG() << "\tfilter --->" << *s << endl;
 			m_inprogress = false;
 			m_kilecompletion = true;
 		}
@@ -1020,29 +1153,64 @@ namespace KileDocument
 
 	void CodeCompletion::slotCharactersInserted(int, int, const QString& string )
 	{
+		KILE_DEBUG() << "==slotCharactersInserted (m_kilecompletion=" << m_kilecompletion << "m_inprogress," << m_inprogress << ",m_ref=" << m_ref << ",string=" << string << ")=============" << endl;
+
+		if ( !inProgress() && m_autoDollar && string=="$" )
+		{
+			autoInsertDollar();
+			return;
+		}
+
 		// only work, if autocomplete mode of Kile is active
 		if ( !isActive() || !autoComplete() )
 			return ;
 
-		//kdDebug() << "==slotCharactersInserted (" << m_kilecompletion << "," << m_inprogress << ", " << string << ")=============" << endl;
 		//FIXME this is not very efficient
-		m_view = info()->viewManager()->currentView();
-		
+		m_view = info()->viewManager()->currentTextView();
+
+		// try to autocomplete abbreviations after punctuation symbol
+		if ( !inProgress() && m_autocompleteabbrev && completeAutoAbbreviation(string) )
+			return;
+
+		// rather unsusual, but it may happen: the cursor is inside
+		// of a reference command without a labellist.
+		if ( ! m_ref )
+		{
+			QString startpattern;
+			CodeCompletion::Type reftype = insideReference(startpattern);
+			if ( reftype != CodeCompletion::ctNone )
+			{
+				m_ref = true;
+				editCompleteList(reftype,startpattern);
+				return;
+			}
+		} // also unusual but may also happen, 20.11.2008 tbraun
+		else if( m_ref && !inProgress())
+		{
+			QString startpattern;
+			CodeCompletion::Type reftype = insideReference(startpattern);
+			if ( reftype == CodeCompletion::ctNone )
+			{
+				m_ref = false;
+				return;
+			}
+		}
+
 		QString word;
 		Type type;
-		bool found = ( m_ref ) ? getReferenceWord(word) : getCompleteWord(true,word,type ); 
-		if ( found ) 
-		{ 
+		bool found = ( m_ref ) ? getReferenceWord(word) : getCompleteWord(true,word,type );
+		if ( found )
+		{
 			int wordlen = word.length();
-			//kdDebug() << "   auto completion: word=" << word << " mode=" << m_mode << " inprogress=" << inProgress() << endl;
+			KILE_DEBUG() << "   auto completion: word=" << word << " mode=" << m_mode << " inprogress=" << inProgress() << endl;
 			if ( inProgress() )               // continue a running mode?
 			{
-				//kdDebug() << "   auto completion: continue current mode" << endl;
-				completeWord(word, m_mode);     
+				KILE_DEBUG() << "   auto completion: continue current mode" << endl;
+				completeWord(word, m_mode);
 			}
 			else if ( word.at( 0 )=='\\' && m_autocomplete && wordlen>=m_latexthreshold)
 			{
-				//kdDebug() << "   auto completion: latex mode" << endl;
+				KILE_DEBUG() << "   auto completion: latex mode" << endl;
 				if ( string.at( 0 ).isLetter() )
 				{
 					completeWord(word, cmLatex);
@@ -1051,10 +1219,10 @@ namespace KileDocument
 				{
 					editCompleteList(type);
 				}
-			} 
-			else if ( word.at(0).isLetter() && m_autocompletetext && wordlen>=m_textthreshold) 
+			}
+			else if ( word.at(0).isLetter() && m_autocompletetext && wordlen>=m_textthreshold)
 			{
-				//kdDebug() << "   auto completion: document mode" << endl;
+				KILE_DEBUG() << "   auto completion: document mode" << endl;
 				completeWord(word,cmDocumentWord);
 			}
 		}
@@ -1125,30 +1293,30 @@ namespace KileDocument
 		// get current text line
 		QString textline = m_view->getDoc()->textLine( row );
 
-		// search the current reference string 
+		// search the current reference string
 		int pos = textline.findRev(reNotRefChars,col-1);
 		if ( pos < 0 )
 			pos = 0;
-			
+
 		// select pattern
 		text = textline.mid(pos+1,col-1-pos);
 		return ( (uint)pos < col-1 );
 	}
-	
+
 	void CodeCompletion::getDocumentWords(const QString &text,
 	                                      QValueList<KTextEditor::CompletionEntry> &list)
 	{
-		//kdDebug() << "getDocumentWords: " << endl;
+		KILE_DEBUG() << "getDocumentWords: " << endl;
 		list.clear();
-		
-		QRegExp reg("(\\\\?\\b" + QString(text[0]) + "[^\\W\\d_]+)\\b");          
+
+		QRegExp reg("(\\\\?\\b" + QString(text[0]) + "[^\\W\\d_]+)\\b");
 		Kate::Document *doc = m_view->getDoc();
-		
+
 		QString s;
 		KTextEditor::CompletionEntry e;
-		QDict<bool> seen; 
+		QDict<bool> seen;
 		bool alreadyseen = true;
-		               
+
 		for (uint i=0; i<doc->numLines(); ++i) {
 			s = doc->textLine(i);
 			int pos = 0;
@@ -1179,6 +1347,129 @@ namespace KileDocument
 		}
 		return ( n % 2 ) ? true : false;
 	}
+
+	//////////////////// complete auto abbreviation ////////////////////
+
+	bool CodeCompletion::completeAutoAbbreviation(const QString &text)
+	{
+		if ( text.length() != 1 )
+			return false;
+
+		QChar ch = text[0];
+		if ( ! (ch.isSpace() || ch.isPunct()) )
+			return false;
+
+		uint row,col;
+		m_view->cursorPositionReal( &row, &col );
+		QString abbrev = getAbbreviationWord(row,col-1);
+		if ( abbrev.isEmpty() )
+			return false;
+
+		QString expansion = findExpansion(abbrev);
+		if ( expansion.isEmpty() )
+			return false;
+
+		KILE_DEBUG() << "=== CodeCompletion::completeAutoAbbreviation: abbrev=" << abbrev << "  exp=" << expansion << endl;
+
+		uint len = abbrev.length();
+		uint startcol = col - len - 1;
+		Kate::Document *doc = m_view->getDoc();
+		doc->removeText( row,startcol,row,startcol+abbrev.length()+1 );
+		doc->insertText( row,startcol,expansion+ch);
+		m_view->setCursorPositionReal( row,startcol+expansion.length()+1 );
+
+		return true;
+	}
+
+	QString CodeCompletion::getAbbreviationWord(uint row, uint col)
+	{
+		QString textline = m_view->getDoc()->textLine( row );
+
+		int index = (int)col;
+		while ( --index >= 0 )
+		{
+			QChar ch = textline.at( index );
+			if ( ! ch.isLetterOrNumber() )
+				break;
+		}
+
+		return textline.mid(index+1,col-index-1);
+	}
+
+	//////////////////// connection with the abbreviation listview  ////////////////////
+
+	void CodeCompletion::setAbbreviationListview(KileAbbrevView *listview)
+	{
+		m_abbrevListview = listview;
+
+		connect( m_abbrevListview, SIGNAL(updateAbbrevList(const QString &, const QString &)),
+		         this, SLOT(slotUpdateAbbrevList(const QString &, const QString &)) );
+
+	}
+
+	void CodeCompletion::slotUpdateAbbrevList(const QString &ds, const QString &as)
+	{
+		if ( ! ds.isEmpty() )
+		{
+			deleteAbbreviationEntry(ds);
+		}
+		if ( ! as.isEmpty() )
+		{
+			addAbbreviationEntry(as);
+		}
+	}
+
+	void CodeCompletion::deleteAbbreviationEntry( const QString &entry )
+	{
+		KILE_DEBUG() << "=== CodeCompletion::deleteAbbreviationEntry (" << entry << ")" << endl;
+		QValueList<KTextEditor::CompletionEntry>::Iterator it;
+		for ( it=m_abbrevlist.begin(); it!=m_abbrevlist.end(); ++it )
+		{
+			if ( (*it).text == entry )
+			{
+				m_abbrevlist.remove( it );
+				return;
+			}
+		}
+	}
+
+	void CodeCompletion::addAbbreviationEntry( const QString &entry )
+	{
+		KILE_DEBUG() << "=== CodeCompletion::addAbbreviationEntry (" << entry << ")" << endl;
+		QValueList<KTextEditor::CompletionEntry>::Iterator it;
+		for ( it=m_abbrevlist.begin(); it!=m_abbrevlist.end(); ++it )
+		{
+			if ( (*it).text > entry )
+				break;
+		}
+
+		KTextEditor::CompletionEntry e;
+		e.text = entry;
+		e.type = QString::null;
+
+		if ( it == m_abbrevlist.begin() )
+			m_abbrevlist.prepend(e);
+		else if ( it == m_abbrevlist.end() )
+			m_abbrevlist.append(e);
+		else
+			m_abbrevlist.insert(it,e);
+	}
+
+	//////////////////// autoinsert $ ////////////////////
+
+	void CodeCompletion::autoInsertDollar()
+	{
+		Kate::View *view = info()->viewManager()->currentTextView();
+		if ( view )
+		{
+			uint row,col;
+			view = info()->viewManager()->currentTextView();
+			view->cursorPositionReal( &row, &col );
+			view->getDoc()->insertText(row,col,"$");
+			view->setCursorPositionReal( row, col );
+		}
+	}
+
 }
 
 #include "codecompletion.moc"
