@@ -1,9 +1,11 @@
 /***************************************************************************
     begin                : Sun Aug 3 2003
     copyright            : (C) 2003 by Jeroen Wijnhout
+                         : (C) 2007  by Holger Danielsson
     email                : Jeroen.Wijnhout@kdemail.net
- ***************************************************************************/
-
+                           holger.danielsson@versanet.de
+***************************************************************************/
+ 
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -13,62 +15,94 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <qlayout.h>
+// 2007-02-15 (dani)
+//  - cosmetic changes
+//  - use of groupboxes to prepare further extensions
+
+// 2007-03-12 (dani)
+//  - use KileDocument::Extensions
+//  - allowed extensions are always defined as list, f.e.: .tex .ltx .latex 
+
+#include "kileprojectdlgs.h"
+
 #include <qlabel.h>
-#include <qcheckbox.h>
 #include <qwhatsthis.h>
 #include <qfileinfo.h>
 #include <qptrlist.h>
+#include <qregexp.h>
+#include <qvalidator.h>
 
 #include <klocale.h>
-#include <klineedit.h>
-#include <kpushbutton.h>
 #include <kmessagebox.h>
 #include <kurlcompletion.h>
 #include <kfiledialog.h>
 #include <kcombobox.h>
-#include <kdebug.h>
+#include "kiledebug.h"
 #include <kapplication.h>
+#include <kiconloader.h>
 
-#include "newfilewizard.h"
 #include "kileproject.h"
-#include "kileprojectdlgs.h"
 #include "kiletoolmanager.h"
 #include "kiledocumentinfo.h"
 #include "kileconfig.h"
+#include "kileextensions.h"
+#include "templates.h"
 
 const QString whatsthisName = i18n("Insert a short descriptive name of your project here.");
 const QString whatsthisPath = i18n("Insert the path to your project file here. If this file does not yet exists, it will be created. The filename should have the extension: .kilepr. You can also use the browse button to insert a filename.");
-const QString whatsthisExt = i18n("Insert a list (separated with spaces) of the extensions of the files in your project that are not TeX source files. These files will be put in a separate place in the Project View. You can also use a regular expression to detect which files are non-source files.");
+const QString whatsthisExt = i18n("Insert a list (separated by spaces) of file extensions which should be treated also as files of the corresponding type in this project.");
 const QString whatsthisMaster = i18n("Select the default master document. Leave empty for auto detection.");
 
 const QString tool_default = i18n("(use global setting)");
 
-KileProjectDlgBase::KileProjectDlgBase(const QString &caption, QWidget *parent, const char * name)
+KileProjectDlgBase::KileProjectDlgBase(const QString &caption, KileDocument::Extensions *extensions, QWidget *parent, const char * name)
 	: KDialogBase( KDialogBase::Plain, caption, (Ok | Cancel), Ok, parent, name, true, true),
-	m_project(0)
+	m_extmanager(extensions), m_project(0)
 {
-	m_title = new KLineEdit(plainPage(), "le_projectname");
+	// properties groupbox
+	m_pgroup = new QVGroupBox(i18n("Project"), plainPage());
+	m_pgroup->setColumnLayout(0, Qt::Vertical );
+	m_pgroup->layout()->setSpacing( 6 );
+	m_pgroup->layout()->setMargin( 11 );
+	m_pgrid = new QGridLayout( m_pgroup->layout() );
+	m_pgrid->setAlignment( Qt::AlignTop );
+
+	m_title = new KLineEdit(m_pgroup, "le_projectname");
 	QWhatsThis::add(m_title, whatsthisName);
+	m_plabel = new QLabel(i18n("Project &title:"), m_pgroup);
+	m_plabel->setBuddy(m_title);
+	QWhatsThis::add(m_plabel, whatsthisName);
+	
+	// extensions groupbox
+	m_egroup= new QVGroupBox(i18n("Extensions"), plainPage());
+	m_egroup->setColumnLayout(0, Qt::Vertical );
+	m_egroup->layout()->setSpacing( 6 );
+	m_egroup->layout()->setMargin( 11 );
+	m_egrid = new QGridLayout( m_egroup->layout() );
+	m_egrid->setAlignment( Qt::AlignTop );
+
+	m_extensions = new KLineEdit(m_egroup, "le_ext");
+	QRegExp reg("[\\. a-zA-Z0-9]+");
+	QRegExpValidator *extValidator = new QRegExpValidator(reg,m_egroup);
+	m_extensions->setValidator(extValidator);
+
+	m_sel_extensions = new KComboBox(false, m_egroup, "le_sel_ext");
+	m_sel_extensions->insertItem(i18n("Source Files"));
+	m_sel_extensions->insertItem(i18n("Package Files"));
+	m_sel_extensions->insertItem(i18n("Image Files"));
+	m_lbPredefinedExtensions = new QLabel(i18n("Predefined:"),m_egroup);
+	m_lbStandardExtensions = new QLabel(QString::null,m_egroup);
 
-	m_extensions = new KLineEdit(plainPage(), "le_ext");
-	m_sel_extensions = new KComboBox(false, plainPage(), "le_sel_ext");
-	m_sel_extensions->insertItem(i18n("Extensions for Source Files"));
-	m_sel_extensions->insertItem(i18n("Extensions for Package Files"));
-	m_sel_extensions->insertItem(i18n("Extensions for Image Files"));
-	m_isregexp = new QCheckBox(i18n("Use extension list as a regular expression"), plainPage());
 	QWhatsThis::add(m_sel_extensions, whatsthisExt);
 	QWhatsThis::add(m_extensions, whatsthisExt);
-	QWhatsThis::add(m_isregexp, whatsthisExt);
+
+	fillProjectDefaults();
 
 	connect(m_sel_extensions, SIGNAL(highlighted(int)),
 		this, SLOT(slotExtensionsHighlighted(int)));
 
 	connect(m_extensions, SIGNAL(textChanged(const QString&)),
 		this, SLOT(slotExtensionsTextChanged(const QString&)));
-
-	connect(m_isregexp, SIGNAL(toggled(bool)),
-		this, SLOT(slotRegExpToggled(bool)));
 }
 
 KileProjectDlgBase::~KileProjectDlgBase()
@@ -83,11 +117,7 @@ void KileProjectDlgBase::slotExtensionsHighlighted(int index)
 	connect(m_extensions, SIGNAL(textChanged(const QString&)),
 		this, SLOT(slotExtensionsTextChanged(const QString&)));
 
-	disconnect(m_isregexp, SIGNAL(toggled(bool)),
-		this, SLOT(slotRegExpToggled(bool)));
-	m_isregexp->setChecked(m_val_isregexp[index]);
-	connect(m_isregexp, SIGNAL(toggled(bool)),
-		this, SLOT(slotRegExpToggled(bool)));
+	m_lbStandardExtensions->setText( m_val_standardExtensions[index] );
 }
 
 void KileProjectDlgBase::slotExtensionsTextChanged(const QString &text)
@@ -95,9 +125,30 @@ void KileProjectDlgBase::slotExtensionsTextChanged(const QString &text)
 	m_val_extensions[m_sel_extensions->currentItem()] = text;
 }
 
-void KileProjectDlgBase::slotRegExpToggled(bool on)
+bool KileProjectDlgBase::acceptUserExtensions()
 {
-	m_val_isregexp[m_sel_extensions->currentItem()] = on;
+	QRegExp reg("\\.\\w+");
+
+	for (int i=KileProjectItem::Source; i<KileProjectItem::Other; ++i) 
+	{
+		m_val_extensions[i-1] = m_val_extensions[i-1].stripWhiteSpace();
+		if ( ! m_val_extensions[i-1].isEmpty() )
+		{
+			// some tiny extension checks
+			QStringList::ConstIterator it;
+			QStringList list = QStringList::split(" ", m_val_extensions[i-1]);
+			for ( it=list.begin(); it != list.end(); ++it ) 
+			{
+				if ( ! reg.exactMatch(*it) )
+				{
+					KMessageBox::error(this, i18n("Error in extension") + " '" + (*it) + "':\n" + i18n("All user defined extensions should look like '.xyz'"), i18n("Invalid extension"));
+					return false;
+				}
+			}
+		}
+	}
+
+	return true;
 }
 
 void KileProjectDlgBase::setExtensions(KileProjectItem::Type type, const QString & ext)
@@ -108,14 +159,6 @@ void KileProjectDlgBase::setExtensions(KileProjectItem::Type type, const QString
 		m_val_extensions[type-1] = ext;
 }
 
-void KileProjectDlgBase::setExtIsRegExp(KileProjectItem::Type type, bool is)
-{
-	if (m_sel_extensions->currentItem() == type-1)
-		m_isregexp->setChecked(is);
-	else
-		m_val_isregexp[type-1] = is;
-}
-
 void KileProjectDlgBase::setProject(KileProject *project, bool override)
 {
 	m_project = project;
@@ -123,16 +166,14 @@ void KileProjectDlgBase::setProject(KileProject *project, bool override)
 	if ((!override) || (project == 0))
 		return;
 
-	for (int i = KileProjectItem::Source; i < KileProjectItem::Other; ++i) {
-		m_val_extensions[i - 1] =
-			project->extensions((KileProjectItem::Type) i);
-		m_val_isregexp[i - 1] =
-			project->extIsRegExp((KileProjectItem::Type) i);
+	for (int i = KileProjectItem::Source; i < KileProjectItem::Other; ++i) 
+	{
+		m_val_extensions[i - 1] = project->extensions((KileProjectItem::Type) i);
 	}
 
 	m_title->setText(m_project->name());
 	m_extensions->setText(m_val_extensions[0]);
-	m_isregexp->setChecked(m_val_isregexp[0]);
+	m_lbStandardExtensions->setText( m_val_standardExtensions[0] );
 }
 
 KileProject* KileProjectDlgBase::project()
@@ -142,70 +183,91 @@ KileProject* KileProjectDlgBase::project()
 
 void KileProjectDlgBase::fillProjectDefaults()
 {
-	m_val_isregexp[0] = false;
-	m_val_isregexp[1] = false;
-	m_val_isregexp[2] = false;
-	//m_val_isregexp[3] = false;
-
-	m_val_extensions[0] = SOURCE_EXTENSIONS;
-	m_val_extensions[1] = PACKAGE_EXTENSIONS;
-	m_val_extensions[2] = IMAGE_EXTENSIONS;
+	m_val_extensions[0] = QString::null;
+	m_val_extensions[1] = QString::null;
+	m_val_extensions[2] = QString::null;
 	//m_val_extensions[3] = OTHER_EXTENSIONS;
 
+	m_val_standardExtensions[0] = m_extmanager->latexDocuments();
+	m_val_standardExtensions[1] = m_extmanager->latexPackages();
+	m_val_standardExtensions[2] = m_extmanager->images();
+
 	m_extensions->setText(m_val_extensions[0]);
-	m_isregexp->setChecked(m_val_isregexp[0]);
+	m_lbStandardExtensions->setText( m_val_standardExtensions[0] );
 }
 
 
 /*
  * KileNewProjectDlg
  */
-KileNewProjectDlg::KileNewProjectDlg(QWidget* parent, const char* name)
-        : KileProjectDlgBase( i18n("Create New Project"), parent, name),
+KileNewProjectDlg::KileNewProjectDlg(KileTemplate::Manager *templateManager, KileDocument::Extensions *extensions, QWidget* parent, const char* name)
+        : KileProjectDlgBase(i18n("Create New Project"), extensions, parent, name), m_templateManager(templateManager),
 		m_filename(QString::null)
 {
-	QGridLayout *layout = new QGridLayout(plainPage(), 4,8, 10);
-	layout->setColStretch(2,1);
-	layout->setColStretch(3,1);
-
-	QLabel *lb = new QLabel(i18n("Project &title:"), plainPage());
-	lb->setBuddy(m_title);
-	QWhatsThis::add(lb, whatsthisName);
-	layout->addWidget(lb, 0,0);
-	layout->addWidget(m_title, 0,1);
+	// Layout
+	QVBoxLayout *vbox = new QVBoxLayout(plainPage(), 6,6 );
 
+	// first groupbox
+	m_pgrid->addWidget(m_plabel, 0,0);
+	m_pgrid->addWidget(m_title, 0,2);
 	connect(m_title, SIGNAL(textChanged(const QString&)), this, SLOT(makeProjectPath()));
 
-	m_location = new KLineEdit(plainPage(), "le_projectlocation");
+	m_location = new KLineEdit(m_pgroup, "le_projectlocation");
 	m_location->setMinimumWidth(200);
 
-	lb = new QLabel(i18n("Project &file:"), plainPage());
-	QWhatsThis::add(lb, whatsthisPath);
+	QLabel *lb1 = new QLabel(i18n("Project &file:"), m_pgroup);
+	QWhatsThis::add(lb1, whatsthisPath);
 	QWhatsThis::add(m_location, whatsthisPath);
-	lb->setBuddy(m_location);
-	KPushButton *pb = new KPushButton(i18n("Select Folder..."), plainPage());
-	connect(pb, SIGNAL(clicked()), this, SLOT(browseLocation()));
-	layout->addWidget(lb, 1,0);
-	layout->addMultiCellWidget(m_location, 1,1, 1,2);
-	layout->addWidget(pb, 1,3);
+	lb1->setBuddy(m_location);
+	m_pbChooseDir= new KPushButton(i18n("Select Folder..."), m_pgroup, "dirchooser_button" );
+	m_pbChooseDir->setPixmap( SmallIcon("fileopen") );
+	int wpixmap = m_pbChooseDir->pixmap()->width();
+	m_pbChooseDir->setFixedWidth(wpixmap+10);
+	m_pbChooseDir->setFixedHeight(wpixmap+10);
 
-	m_cb = new QCheckBox(i18n("Create a new file and add it to this project"),plainPage());
+	m_pgrid->addWidget(lb1,1,0);
+	m_pgrid->addMultiCellWidget(m_location,1,1,2,3);
+	m_pgrid->addWidget(m_pbChooseDir,1,5);
+	m_pgrid->setColSpacing(1,8);
+	m_pgrid->setColSpacing(4,8);
+	m_pgrid->setColStretch(3,1);
+
+	connect(m_pbChooseDir, SIGNAL(clicked()), this, SLOT(browseLocation()));
+
+	// second groupbox
+	QVGroupBox* group2= new QVGroupBox(i18n("File"), plainPage());
+	QWidget *widget2 = new QWidget(group2);
+	QGridLayout *grid2 = new QGridLayout(widget2, 3,2, 6,6);
+	m_cb = new QCheckBox(i18n("Create a new file and add it to this project"),widget2);
 	m_cb->setChecked(true);
-	m_lb  = new QLabel(i18n("File&name (relative to where the project file is):"), plainPage());
-	m_file = new KLineEdit(plainPage());
+	m_lb  = new QLabel(i18n("File&name (relative to where the project file is):"),widget2);
+	m_file = new KLineEdit(widget2);
 	m_lb->setBuddy(m_file);
-	m_nfw = new NewFileWidget(plainPage());
+	m_templateIconView = new TemplateIconView(widget2);
+	m_templateIconView->setTemplateManager(m_templateManager);
+	m_templateManager->scanForTemplates();
+	m_templateIconView->fillWithTemplates(KileDocument::LaTeX);
 	QWhatsThis::add(m_cb, i18n("If you want Kile to create a new file and add it to the project, then check this option and select a template from the list that will appear below."));
-	layout->addMultiCellWidget(m_cb, 2,2, 0,3);
-	layout->addMultiCellWidget(m_lb, 3,3, 0,1);
-	layout->addMultiCellWidget(m_file, 3,3, 2,3);
-	layout->addMultiCellWidget(m_nfw, 4,4, 0,3);
+
+	grid2->addMultiCellWidget(m_cb, 0,0, 0,1);
+	grid2->addWidget(m_lb, 1,0);
+	grid2->addWidget(m_file, 1,1);
+	grid2->addMultiCellWidget(m_templateIconView, 2,2, 0,1);
+	grid2->setColStretch(1,1);
 	connect(m_cb, SIGNAL(clicked()), this, SLOT(clickedCreateNewFileCb()));
 
-	layout->addWidget(m_sel_extensions, 6,0);
-	layout->addMultiCellWidget(m_extensions, 6,6, 1,3);
-	layout->addMultiCellWidget(m_isregexp, 7,7, 1,3);
+	// third groupbox
+	m_egrid->addWidget(m_sel_extensions, 6,0);
+	m_egrid->addMultiCellWidget(m_extensions, 6,6, 1,3);
+	m_egrid->addWidget(m_lbPredefinedExtensions, 7,0);
+	m_egrid->addMultiCellWidget(m_lbStandardExtensions, 7,7, 1,3);
 
+	// add to layout
+	vbox->addWidget(m_pgroup);
+	vbox->addWidget(group2);
+	vbox->addWidget(m_egroup);
+	vbox->addStretch();
+	
 	fillProjectDefaults();
 }
 
@@ -215,12 +277,11 @@ KileNewProjectDlg::~KileNewProjectDlg()
 KileProject* KileNewProjectDlg::project()
 {
 	if (!m_project) {
-		m_project = new KileProject(projectTitle(), KURL(location()));
+		m_project = new KileProject(projectTitle(), KURL(location()), m_extmanager);
 
 		KileProjectItem::Type type;
 		for (int i = KileProjectItem::Source; i < KileProjectItem::Other; ++i) {
 			type = (KileProjectItem::Type) i;
-			m_project->setExtIsRegExp(type, extIsRegExp(type));
 			m_project->setExtensions(type, extensions(type));
 		}
 
@@ -236,13 +297,13 @@ void KileNewProjectDlg::clickedCreateNewFileCb()
 	{
 		m_file->show();
 		m_lb->show();
-		m_nfw->show();
+		m_templateIconView->show();
 	}
 	else
 	{
 		m_file->hide();
 		m_lb->hide();
-		m_nfw->hide();
+		m_templateIconView->hide();
 	}
 }
 
@@ -262,7 +323,7 @@ void KileNewProjectDlg::browseLocation()
 	if (! dir.isNull())
 	{
 		m_dir = dir;
-		if (m_dir.right(1) != "/") m_dir = m_dir+"/";
+		if (m_dir.right(1) != "/") m_dir = m_dir + '/';
 		m_location->setText(m_dir+bare());
 	}
 }
@@ -270,16 +331,19 @@ void KileNewProjectDlg::browseLocation()
 void KileNewProjectDlg::makeProjectPath()
 {
 	m_filename=bare();
-	kdDebug() << "BEFORE " << QFileInfo(location()).absFilePath() << " " << QFileInfo(location()).dirPath() << endl;
+	KILE_DEBUG() << "BEFORE " << QFileInfo(location()).absFilePath() << " " << QFileInfo(location()).dirPath() << endl;
 	m_dir = QFileInfo(location()).dirPath();
-	if (m_dir.right(1) != "/") m_dir = m_dir+"/";
+	if (m_dir.right(1) != "/") m_dir = m_dir + '/';
 
-	kdDebug() << "LOCATION " << location() << " AND " << m_dir << endl;
+	KILE_DEBUG() << "LOCATION " << location() << " AND " << m_dir << endl;
 	m_location->setText(m_dir+m_filename);
 }
 
 void KileNewProjectDlg::slotOk()
 {
+	if ( ! acceptUserExtensions() )
+		return;
+
 	//replace ~ and environment variables in the paths
 	KURLCompletion uc;
 	uc.setReplaceEnv(true);
@@ -319,8 +383,8 @@ void KileNewProjectDlg::slotOk()
 			return;
 		}
 
-		kdDebug() << "==KileNewProjectDlg::slotOk()==============" << endl;
-		kdDebug() << "\t" << location() << " " << fi.dirPath() << endl;
+		KILE_DEBUG() << "==KileNewProjectDlg::slotOk()==============" << endl;
+		KILE_DEBUG() << "\t" << location() << " " << fi.dirPath() << endl;
 		if (! dr.exists())
 		{
 			bool suc = true;
@@ -329,14 +393,14 @@ void KileNewProjectDlg::slotOk()
 
 			for (uint i=0; i < dirs.count(); ++i)
 			{
-				path += "/"+dirs[i];
+				path += '/' + dirs[i];
 				dir.setPath(path);
-				kdDebug() << "\tchecking : " << dir.absPath() << endl;
+				KILE_DEBUG() << "\tchecking : " << dir.absPath() << endl;
 				if ( ! dir.exists() )
 				{
 					dir.mkdir(dir.absPath());
 					suc = dir.exists();
-					kdDebug() << "\t\tcreated : " << dir.absPath() << " suc = " << suc << endl;
+					KILE_DEBUG() << "\t\tcreated : " << dir.absPath() << " suc = " << suc << endl;
 				}
 
 				if (!suc)
@@ -365,7 +429,7 @@ void KileNewProjectDlg::slotOk()
 
 		//check for validity of name first, then check for existence (fixed by tbraun)
 		KURL fileURL; fileURL.setFileName(file());
-		KURL validURL = KileDocument::Info::makeValidTeXURL(fileURL);
+		KURL validURL = KileDocument::Info::makeValidTeXURL(fileURL,m_extmanager->isTexFile(fileURL),true);
 		if ( validURL != fileURL )
 			m_file->setText(validURL.fileName());
 
@@ -388,8 +452,10 @@ void KileNewProjectDlg::slotOk()
 
 void KileNewProjectDlg::fillProjectDefaults()
 {
-	m_dir = KileConfig::defaultProjectLocation() + "/";
-	kdDebug() << "M_DIR " << m_dir << endl;
+	m_dir = KileConfig::defaultProjectLocation();
+	if( !m_dir.endsWith("/") )
+		m_dir += '/';
+	KILE_DEBUG() << "M_DIR " << m_dir << endl;
 	m_location->setText(m_dir);
 	m_cb->setChecked(true);
 
@@ -398,35 +464,46 @@ void KileNewProjectDlg::fillProjectDefaults()
 
 TemplateItem* KileNewProjectDlg::getSelection() const
 {
-	return static_cast<TemplateItem*>(m_nfw->currentItem());
+	return static_cast<TemplateItem*>(m_templateIconView->currentItem());
 }
 
 /*
  * KileProjectOptionsDlg
  */
-KileProjectOptionsDlg::KileProjectOptionsDlg(KileProject *project, QWidget *parent, const char * name) :
- 	KileProjectDlgBase(i18n("Project Options"), parent, name )
+KileProjectOptionsDlg::KileProjectOptionsDlg(KileProject *project, KileDocument::Extensions *extensions, QWidget *parent, const char * name) :
+ 	KileProjectDlgBase(i18n("Project Options"), extensions, parent, name )
 {
-	QGridLayout *layout = new QGridLayout(plainPage(), 5,4, 10);
+	// Layout
+	QVBoxLayout *vbox = new QVBoxLayout(plainPage(), 6,6 );
 
-	QLabel *lb = new QLabel(i18n("Project &title:"), plainPage());
-	lb->setBuddy(m_title);
-	QWhatsThis::add(lb, whatsthisName);
-	layout->addWidget(lb, 0,0);
-	layout->addMultiCellWidget(m_title, 0,0, 1,3);
+	m_pgrid->addWidget(m_plabel, 0,0);
+	m_pgrid->addWidget(m_title, 0,2);
+//	m_pgrid->addWidget(labelEncoding, 1,0);
+//	m_pgrid->addWidget(m_lbEncoding, 1,2);
+	m_pgrid->setColSpacing(1,8);
+	m_pgrid->setColStretch(3,1);
+	// second groupbox
 
-	layout->addWidget(m_sel_extensions, 2,0);
-	layout->addMultiCellWidget(m_extensions, 2,2, 1,3);
-	layout->addMultiCellWidget(m_isregexp, 3,3, 1,3);
+	m_egrid->addWidget(m_sel_extensions, 6,0);
+	m_egrid->addMultiCellWidget(m_extensions, 6,6, 1,3);
+	m_egrid->addWidget(m_lbPredefinedExtensions, 7,0);
+	m_egrid->addMultiCellWidget(m_lbStandardExtensions, 7,7, 1,3);
 
-	m_master = new KComboBox(false, plainPage(), "master");
+	// third groupbox
+	QVGroupBox* group3 = new QVGroupBox(i18n("Properties"), plainPage());
+	group3->setColumnLayout(0, Qt::Vertical );
+	group3->layout()->setSpacing( 6 );
+	group3->layout()->setMargin( 11 );
+	QGridLayout *grid3 = new QGridLayout( group3->layout() );
+	grid3->setAlignment( Qt::AlignTop );
+
+	m_master = new KComboBox(false, group3, "master");
 	//m_master->setDisabled(true);
-	lb = new QLabel(i18n("&Master document:"), plainPage());
-	lb->setBuddy(m_master);
+	QLabel *lb1 = new QLabel(i18n("&Master document:"), group3);
+	lb1->setBuddy(m_master);
+	lb1->setMinimumWidth( m_sel_extensions->sizeHint().width() );
 	QWhatsThis::add(m_master, whatsthisMaster);
-	QWhatsThis::add(lb,whatsthisMaster);
-	layout->addWidget(lb, 4,0);
-	layout->addMultiCellWidget(m_master, 4,4, 1,3);
+	QWhatsThis::add(lb1,whatsthisMaster);
 
 	m_master->insertItem(i18n("(auto-detect)"));
 	QPtrListIterator<KileProjectItem> rit(*(project->rootItems()));
@@ -446,9 +523,9 @@ KileProjectOptionsDlg::KileProjectOptionsDlg(KileProject *project, QWidget *pare
 	if (project->masterDocument().isNull())
 		m_master->setCurrentItem(0);
 
-	lb = new QLabel(i18n("&QuickBuild configuration:"), plainPage()); layout->addWidget(lb, 5, 0);
-	m_cbQuick = new KComboBox(plainPage()); layout->addWidget(m_cbQuick, 5, 1);
-	lb->setBuddy(m_cbQuick);
+	QLabel *lb2 = new QLabel(i18n("&QuickBuild configuration:"), group3); 
+	m_cbQuick = new KComboBox(group3); 
+	lb2->setBuddy(m_cbQuick);
 	m_cbQuick->insertItem(tool_default);
 	m_cbQuick->insertStringList(KileTool::configNames("QuickBuild", kapp->config()));
 	m_cbQuick->setCurrentText(project->quickBuildConfig().length() > 0 ? project->quickBuildConfig() : tool_default );
@@ -456,11 +533,25 @@ KileProjectOptionsDlg::KileProjectOptionsDlg(KileProject *project, QWidget *pare
 	//don't put this after the call to toggleMakeIndex
 	setProject(project, true);
 
-	m_ckMakeIndex = new QCheckBox(i18n("&MakeIndex options"), plainPage()); layout->addWidget(m_ckMakeIndex, 6, 0);
+	m_ckMakeIndex = new QCheckBox(i18n("&MakeIndex options"), group3); 
 	connect(m_ckMakeIndex, SIGNAL(toggled(bool)), this, SLOT(toggleMakeIndex(bool)));
-	m_leMakeIndex = new KLineEdit(plainPage()); layout->addMultiCellWidget(m_leMakeIndex, 6, 6, 1, 3);
+	m_leMakeIndex = new KLineEdit(group3); 
 	m_ckMakeIndex->setChecked(project->useMakeIndexOptions());
 	toggleMakeIndex(m_ckMakeIndex->isChecked());
+
+	grid3->addWidget(lb1,0,0);
+	grid3->addWidget(m_master,0,1);
+	grid3->addWidget(lb2,1,0);
+	grid3->addWidget(m_cbQuick,1,1);
+	grid3->addWidget(m_ckMakeIndex,2,0);
+	grid3->addMultiCellWidget(m_leMakeIndex,2,2,1,2);
+	grid3->setColStretch(2,1);
+
+	// add to layout
+	vbox->addWidget(m_pgroup);
+	vbox->addWidget(m_egroup);
+	vbox->addWidget(group3);
+	vbox->addStretch();
 }
 
 KileProjectOptionsDlg::~KileProjectOptionsDlg()
@@ -469,7 +560,7 @@ KileProjectOptionsDlg::~KileProjectOptionsDlg()
 
 void KileProjectOptionsDlg::toggleMakeIndex(bool on)
 {
-	kdDebug() << "TOGGLED!" << endl;
+	KILE_DEBUG() << "TOGGLED!" << endl;
 	m_leMakeIndex->setEnabled(on);
 	m_project->setUseMakeIndexOptions(on);
 	m_project->writeUseMakeIndexOptions();
@@ -479,6 +570,9 @@ void KileProjectOptionsDlg::toggleMakeIndex(bool on)
 
 void KileProjectOptionsDlg::slotOk()
 {
+	if ( ! acceptUserExtensions() )
+		return;
+
 	this->m_project->setName(m_title->text());
 
 	QPtrListIterator<KileProjectItem> rit(*(m_project->rootItems()));
@@ -490,16 +584,11 @@ void KileProjectOptionsDlg::slotOk()
 	}
 	if (m_master->currentItem() == 0) m_project->setMasterDocument(QString::null);
 
-	m_val_extensions[m_sel_extensions->currentItem()]
-		= m_extensions->text();
-	m_val_isregexp[m_sel_extensions->currentItem()]
-		= m_isregexp->isChecked();
+	m_val_extensions[m_sel_extensions->currentItem()] = m_extensions->text();
 
-	for (int i = KileProjectItem::Source; i < KileProjectItem::Other; ++i) {
-		m_project->setExtensions
-			((KileProjectItem::Type) i, m_val_extensions[i-1]);
-		m_project->setExtIsRegExp
-			((KileProjectItem::Type) i, m_val_isregexp[i-1]);
+	for (int i = KileProjectItem::Source; i < KileProjectItem::Other; ++i) 
+	{
+		m_project->setExtensions( (KileProjectItem::Type) i, m_val_extensions[i-1] );
 	}
 
 	if ( m_cbQuick->currentText() == tool_default )
