@@ -1,9 +1,9 @@
 /***************************************************************************
     begin                : Sun Dec 28 2003
     copyright            : (C) 2003 by Jeroen Wijnhout
-                               2005 by Holger Danielsson
+                               2005-2007  by Holger Danielsson
     email                : Jeroen.Wijnhout@kdemail.net
-                           holger.danielsson@t-online.de
+                           holger.danielsson@versanet.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -37,13 +37,35 @@
 // 2005-12-16: dani
 //  - add listview item for undefined references
 
+// 2007-02-15: dani
+// - class KileListViewItem gets two new members to 
+//   save the real cursor position of the command
+
+// 2007-03-12 dani
+//  - use KileDocument::Extensions
+
+// 2007-03-17 dani
+//  - remember how document structure is collapsed, when structure view is refreshed
+
+// 2007-03-24: dani
+// - preliminary minimal support for Beamer class
+// - \begin{frame}...\end{frame} and \frame{...} are shown in the structure view
+// - if a \frametitle command follows as next LaTeX command, its parameter
+//   is taken to replace the standard title of this entry in the structure view
+// - \begin{block}...\end{block} is taken as child of a frame
+
+// 2007-04-06 dani
+// - add TODO/FIXME section to structure view
+
+#include "kilestructurewidget.h"
+
 #include <qfileinfo.h>
 #include <qheader.h>
 #include <qregexp.h>
 #include <qclipboard.h>
  
 #include <kapplication.h>
-#include <kdebug.h>
+#include "kiledebug.h"
 #include <kmessagebox.h>
 #include <klocale.h>
 #include <kiconloader.h>
@@ -55,14 +77,14 @@
 #include "kileinfo.h"
 #include "kiledocmanager.h"
 #include "kiledocumentinfo.h"
-#include "kilestructurewidget.h"
 #include "kileconfig.h"
 
 ////////////////////// KileListViewItem with all info //////////////////////
 
-KileListViewItem::KileListViewItem(QListViewItem * parent, QListViewItem * after, const QString &title, const KURL &url, uint line, uint column, int type, int level) : 
+KileListViewItem::KileListViewItem(QListViewItem * parent, QListViewItem * after, const QString &title, const KURL &url, uint line, uint column, int type, int level, uint startline, uint startcol) : 
 	KListViewItem(parent,after),
-	m_title(title), m_url(url), m_line(line), m_column(column), m_type(type), m_level(level)
+	m_title(title), m_url(url), m_line(line), m_column(column), m_type(type), m_level(level),
+	m_startline(startline), m_startcol(startcol)
 {
 	setItemEntry();
 }
@@ -85,7 +107,7 @@ void KileListViewItem::setTitle(const QString &title)
 
 void KileListViewItem::setItemEntry()
 {
-	setText(0, m_title + " (" + i18n("line") + " " + QString::number(m_line) + ")");
+	setText(0, m_title + " (" + i18n("line") + ' ' + QString::number(m_line) + ')');
 }
 
 ////////////////////// introduce a new ToolTip //////////////////////
@@ -130,6 +152,8 @@ namespace KileWidget
 		header()->hide();
 		addColumn(i18n("Structure"),-1);
 		setSorting(-1,true);
+		setAllColumnsShowFocus(true);
+		setFullWidth(true);
 		setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored);
 
 		QToolTip::remove(this);
@@ -140,7 +164,8 @@ namespace KileWidget
 		connect(this, SIGNAL(contextMenu(KListView *, QListViewItem *, const QPoint & )), m_stack, SLOT(slotPopup(KListView *, QListViewItem * , const QPoint & )));
 		
 		connect(this, SIGNAL(executed(QListViewItem*)), m_stack, SLOT(slotClicked(QListViewItem*)));
-
+		connect(m_stack, SIGNAL(configChanged()), this, SLOT(slotConfigChanged()));
+		
 		init();
 	}
 
@@ -155,7 +180,8 @@ namespace KileWidget
 			m_root->setURL(m_docinfo->url());
 			m_root->setOpen(true);
 			m_root->setPixmap(0, SmallIcon("contents"));
-			connect(m_docinfo, SIGNAL(foundItem(const QString&, uint, uint, int, int, const QString &, const QString &)), this, SLOT(addItem(const QString&, uint, uint, int, int, const QString &, const QString &)));
+			connect(m_docinfo, SIGNAL(foundItem(const QString&, uint, uint, int, int, uint, uint, const QString &, const QString &)), 
+			        this, SLOT(addItem(const QString&, uint, uint, int, int, uint, uint, const QString &, const QString &)));
 // 			connect(m_docinfo, SIGNAL(doneUpdating()), this, SLOT(insertInMasterList()));
 		}
 
@@ -163,6 +189,8 @@ namespace KileWidget
 		m_lastType = KileStruct::None;
 		m_lastSectioning = 0L;
 		m_lastFloat = 0L;
+		m_lastFrame = 0L;
+		m_lastFrameEnv = 0L;
 		m_stop = false;
 
 		m_folders.clear();
@@ -173,29 +201,51 @@ namespace KileWidget
  			m_openStructureLabels = m_docinfo->openStructureLabels();
  			m_openStructureReferences = m_docinfo->openStructureReferences();
 			m_openStructureBibitems = m_docinfo->openStructureBibitems();
+			m_openStructureTodo = m_docinfo->openStructureTodo();
+			m_showStructureLabels = m_docinfo->showStructureLabels();
 		}
 		else
 		{
 			m_openStructureLabels = false;
 			m_openStructureReferences = false;
 			m_openStructureBibitems = false;
+			m_openStructureTodo = false;
+			m_showStructureLabels = false;
  		}
 	}
 
-	void StructureList::cleanUp()
+	void StructureList::updateRoot() 
 	{
-        kdDebug() << "==void StructureList::cleanUp()========" << endl;
-		saveState();
+		m_root->setURL( m_docinfo->url() ); 
+		m_root->setText(0, m_docinfo->url().fileName() );		
+	}
+
+	void StructureList::cleanUp(bool preserveState/* = true */)
+	{
+        	KILE_DEBUG() << "==void StructureList::cleanUp()========" << endl;
+		if(preserveState)
+			saveState();
 		clear();
-		if(NULL != m_docinfo)
+		if(m_docinfo)
 			disconnect(m_docinfo, 0, this, 0);
 		init();
 	}
+	
+	void StructureList::slotConfigChanged(){
+		QWidget *current = m_stack->visibleWidget();
+		if(!current)
+			return;
+		cleanUp(false);
+ 		m_stack->update(m_docinfo,true,false);
+		m_stack->raiseWidget(current);
+	}
 
 	void StructureList::saveState()
 	{
+		KILE_DEBUG() << "===void StructureList::saveState()" << endl;
 		m_openByTitle.clear();
 		m_openByLine.clear();
+		m_openByFolders.clear();
 
 		QListViewItemIterator it(this);
 		KileListViewItem *item = 0L;
@@ -215,18 +265,51 @@ namespace KileWidget
 			}
 			++it;
 		}
+
+		if ( m_folders.contains("labels") ) 
+			m_openByFolders["labels"] = m_folders["labels"]->isOpen();
+		if ( m_folders.contains("refs") ) 
+			m_openByFolders["refs"] = m_folders["refs"]->isOpen();
+		if ( m_folders.contains("bibs") ) 
+			m_openByFolders["bibs"] = m_folders["bibs"]->isOpen();
+		if ( m_folders.contains("todo") ) 
+			m_openByFolders["todo"] = m_folders["todo"]->isOpen();
+		if ( m_folders.contains("fixme") ) 
+			m_openByFolders["fixme"] = m_folders["fixme"]->isOpen();
 	}
 
 	bool StructureList::shouldBeOpen(KileListViewItem *item, const QString & folder, int level)
 	{
-		if ( item->parent() == 0L ) return true;
-
+		if ( item->parent() == 0L )
+			return true;
 		if ( folder == "labels" )
-			return m_openStructureLabels;
-		if ( folder == "refs" )
-			return m_openStructureReferences;
-		if ( folder == "bibs" )
-			return m_openStructureBibitems;
+		{
+			if ( m_openByFolders.contains("labels") )
+				return m_openByFolders["labels"];
+			else
+				return m_openStructureLabels;
+		}
+		else if ( folder == "refs" )
+		{
+			if ( m_openByFolders.contains("refs") )
+				return m_openByFolders["refs"];
+			else
+				return m_openStructureReferences;
+		}
+		else if ( folder == "bibs" )
+		{
+			if ( m_openByFolders.contains("bibs") )
+				return m_openByFolders["bibs"];
+			else
+				return m_openStructureBibitems;
+		}
+		else if ( folder=="todo" || folder=="fixme" )
+		{
+			if ( m_openByFolders.contains(folder) )
+				return m_openByFolders[folder];
+			else
+				return m_openStructureTodo;
+		}
 
 		if ( m_openByTitle.contains(item->title()) )
 			return m_openByTitle [ item->title() ];
@@ -248,13 +331,23 @@ namespace KileWidget
 		else if ( folder == "bibs" )
 		{
 			fldr->setText(0, i18n("BibTeX References"));
-			fldr->setPixmap(0, SmallIcon("bibtex"));
+			fldr->setPixmap(0, SmallIcon("viewbib"));
 		}
 		else if ( folder == "refs" )
 		{
 			fldr->setText(0, i18n("Undefined References"));
 			fldr->setPixmap(0, SmallIcon("stop"));
 		}
+		else if ( folder == "todo" )
+		{
+			fldr->setText(0, i18n("TODO"));
+			fldr->setPixmap(0, SmallIcon("bookmark"));
+		}
+		else if ( folder == "fixme" )
+		{
+			fldr->setText(0, i18n("FIXME"));
+			fldr->setPixmap(0, SmallIcon("bookmark"));
+		}
 
 		m_folders[ folder ] = fldr;
 
@@ -309,8 +402,8 @@ namespace KileWidget
 	/* some items have a special action:
 		- KileStruct::Sect: 
 		      The new item is saved in m_lastSectioning, so that all following entries 
-		      can be inserted as childs. \part will drop back to level 0 of the Listview,
-		      all other sectioning commands will be childs of the last sectioning item.
+		      can be inserted as children. \part will drop back to level 0 of the Listview,
+		      all other sectioning commands will be children of the last sectioning item.
 		      If a \label command follows in the same or the next line, it is assigned
 		      to this item.
 		- KileStruct::BeginFloat:
@@ -328,12 +421,26 @@ namespace KileWidget
 		      If we are inside a float, this label is assigned to this environment. If the last 
 		      type was a sectioning command on the current line or the line before, the label is 
 		      assigned to this sectioning item. Assigning means that a popup menu will open, 
-		      when the mouse is over this item.  
+		      when the mouse is over this item.
+		- KileStruct::BeamerBeginFrame
+		      The new item is saved in m_lastFrameEnv. If a \frametitle command follows before
+		      the frame environment is closed, it is inserted into the title of this item. 
+		      If a \label command follows, it is assigned to this float item.
+		- KileStruct::BeamerEndFrame
+		      Reset m_lastFloatEnv to 0L to close this environment. No more \frametitle 
+		      or \label commands are assigned to this frame after this.
+		- KileStruct::BeamerBeginBlock
+		      Inside a beamer frame this environment is taken as child of this frame
+		- KileStruct::BeamerFrame
+		      The new item is saved in m_lastFrame. If a \frametitle command follows
+		      immediately as next command, it is inserted into the title of this item. 
 		*/
 		
-	void StructureList::addItem(const QString &title, uint line, uint column, int type, int lev, const QString & pix, const QString & fldr /* = "root" */)
+	void StructureList::addItem(const QString &title, uint line, uint column, int type, int lev,
+	                            uint startline, uint startcol, 
+	                            const QString & pix, const QString &fldr /* = "root" */)
 	{
-//  		kdDebug() << "\t\taddItem: " << title << ", with type " <<  type << endl;
+//  		KILE_DEBUG() << "\t\taddItem: " << title << ", with type " <<  type << endl;
 		if ( m_stop ) return;
 		
 		// some types need a special action
@@ -348,16 +455,30 @@ namespace KileWidget
 				m_lastFloat->setTitle(floattitle+": "+title);
 		}
 		else if ( type == KileStruct::EndFloat )
+		{
 			m_lastFloat = 0L;
+		}
+		else if ( type==KileStruct::BeamerFrametitle )
+		{
+			if (  m_lastFrameEnv )
+				m_lastFrameEnv->setTitle(title);
+			else if (  m_lastFrame )
+				m_lastFrame->setTitle(title);		
+		}
+		else if ( type == KileStruct::BeamerEndFrame )
+		{
+			m_lastFrameEnv = 0L;
+		}
+		m_lastFrame = 0L;
 		
 		// that's all for hidden types: we must immediately return
 		if ( lev == KileStruct::Hidden ) 
 		{
-			//kdDebug() << "\t\thidden item: not created" << endl;
+			//KILE_DEBUG() << "\t\thidden item: not created" << endl;
 			return;
 		}
 			
-		//kdDebug() << "\t\tcreate new item" << endl;
+		//KILE_DEBUG() << "\t\tcreate new item" << endl;
 		// check if we have to update a label before loosing this item
 		if ( type==KileStruct::Label )
 		{
@@ -371,6 +492,12 @@ namespace KileWidget
 				if ( m_lastSectioning && (m_lastLine==line-1 || m_lastLine==line) )
 					m_lastSectioning->setLabel(title);
 			}
+			else if(m_lastType == KileStruct::BeamerBeginFrame && m_lastFrameEnv) {
+				m_lastFrameEnv->setLabel(title);
+			}
+
+			if(!m_showStructureLabels) // if we don't want to have it displayed return here
+					return;
 		}
 		
 		// remember current type and line for the next call of addItem()
@@ -378,7 +505,7 @@ namespace KileWidget
 		m_lastLine = line;
 
 		//find the parent for the new element
-		KileListViewItem *parentItem = parentFor(lev, fldr);
+		KileListViewItem *parentItem = ( type==KileStruct::BeamerBeginBlock && m_lastFrameEnv ) ? m_lastFrameEnv : parentFor(lev, fldr);
 		if ( parentItem == 0L )
 		{
 			KMessageBox::error(0,i18n("Can't create ListviewItem: no parent found."));
@@ -395,7 +522,7 @@ namespace KileWidget
 		}
 		
 		// create a new item
-		KileListViewItem *newChild = new KileListViewItem(parentItem, lastChild, title, m_docinfo->url(), line, column, type, lev);
+		KileListViewItem *newChild = new KileListViewItem(parentItem, lastChild, title, m_docinfo->url(), line, column, type, lev, startline, startcol);
 		if ( ! pix.isNull() ) 
 			newChild->setPixmap(0,SmallIcon(pix));
 		//m_stop = true;
@@ -423,6 +550,10 @@ namespace KileWidget
 			m_lastSectioning = newChild;
 		else if ( type == KileStruct::BeginFloat )
 			m_lastFloat = newChild;
+		else if ( type == KileStruct::BeamerBeginFrame )
+			m_lastFrameEnv = newChild;
+		else if ( type == KileStruct::BeamerFrame )
+			m_lastFrame = newChild;
 	}
 
 	void StructureList::showReferences(KileInfo *ki)
@@ -437,14 +568,14 @@ namespace KileWidget
 			m_folders.remove("refs");
 		}
 
-		//kdDebug() << "==void StructureList::showReferences()========" << endl;
-		//kdDebug() << "\tfound " << m_references.count() << " references" << endl;
+		//KILE_DEBUG() << "==void StructureList::showReferences()========" << endl;
+		//KILE_DEBUG() << "\tfound " << m_references.count() << " references" << endl;
 		if ( m_references.count() == 0 )
 			return;
 			
 		// read list with all labels
 		const QStringList *list = ki->allLabels();
-		//kdDebug() << "\tfound " << list->count() << " labels" << endl;
+		//KILE_DEBUG() << "\tfound " << list->count() << " labels" << endl;
 		QMap<QString,bool> labelmap;
 		for ( QStringList::ConstIterator itmap=list->begin(); itmap!=list->end(); ++itmap ) 
 		{
@@ -458,8 +589,8 @@ namespace KileWidget
 			if ( ! labelmap.contains((*it).name()) )
 			{ 
 				KileListViewItem *refitem = folder("refs");
-				refitem->setOpen(m_openStructureReferences);
-				new KileListViewItem(refitem,0L,(*it).name(),m_docinfo->url(),(*it).line(),(*it).column(),KileStruct::Reference,KileStruct::NotSpecified);
+				refitem->setOpen( shouldBeOpen(refitem,"refs",0) );
+				new KileListViewItem(refitem,0L,(*it).name(),m_docinfo->url(),(*it).line(),(*it).column(),KileStruct::Reference,KileStruct::NotSpecified, 0,0 );
 			}
 		}
 	}
@@ -471,7 +602,7 @@ namespace KileWidget
 		m_ki(ki),
 		m_docinfo(0L)
 	{
-		kdDebug() << "==KileWidget::Structure::Structure()===========" << endl;
+		KILE_DEBUG() << "==KileWidget::Structure::Structure()===========" << endl;
 		setLineWidth(0);
 		setMidLineWidth(0);
 		setMargin(0);
@@ -501,7 +632,7 @@ namespace KileWidget
 
 	void Structure::slotClicked(QListViewItem * itm)
 	{
-		kdDebug() << "\tStructure::slotClicked" << endl;
+		KILE_DEBUG() << "\tStructure::slotClicked" << endl;
 
 		KileListViewItem *item = (KileListViewItem *)itm;
 		//return if user didn't click on an item
@@ -515,30 +646,31 @@ namespace KileWidget
 
 	void Structure::slotDoubleClicked(QListViewItem * itm)
 	{
-		kdDebug() << "\tStructure::slotDoubleClicked" << endl;
+		KILE_DEBUG() << "\tStructure::slotDoubleClicked" << endl;
 		KileListViewItem *item = (KileListViewItem*)(itm);
 		static QRegExp::QRegExp suffix("\\.[\\d\\w]*$");
 		
 		if (!item)
 			return;
 		
-		kdDebug() <<"item->url() is " << item->url() << ", item->title() is " << item->title() << endl;
+		KILE_DEBUG() <<"item->url() is " << item->url() << ", item->title() is " << item->title() << endl;
 		
 		if ( item->type() & (KileStruct::Input | KileStruct::Bibliography | KileStruct::Graphics) )
 		{
-			QString fname = item->title();
-			
+			QString fname = item->title();	
 			
 			if(fname.find(suffix) != -1) // check if we have a suffix, if not add standard suffixes
 			{
-				kdDebug() << "Suffix found: " << suffix.cap(0) << endl;
+				KILE_DEBUG() << "Suffix found: " << suffix.cap(0) << endl;
 			}
 			else
 			{
+				// filename in structureview entry has no extension: this shouldn't happen anymore,
+				// because all have got one in function updateStruct(). But who knows?
 				if ( item->type() == KileStruct::Input )
-					fname += ".tex";
+					fname += m_ki->extensions()->latexDocumentDefault();
 				else if ( item->type() == KileStruct::Bibliography )
-					fname += ".bib";
+					fname += m_ki->extensions()->bibtexDefault();
 				else
 				{
 					// FIXME get graphics suffix from an project config option
@@ -548,7 +680,7 @@ namespace KileWidget
 			if(fname.left(1) != "/") // no absolute path
 			{
 				QString fn = m_ki->getCompileName();
-				fname= QFileInfo(fn).dirPath() + "/" + fname;
+				fname= QFileInfo(fn).dirPath() + '/' + fname;
 			}
 			
 			QFileInfo fi(fname);
@@ -565,53 +697,73 @@ namespace KileWidget
 				else
 					emit(fileOpen(url, QString::null));
 			}
-			else
-			{
-				if ( KMessageBox::warningYesNo(this, i18n("Cannot find the included file. The file does not exist, is not readable or Kile is unable to determine the correct path to it. The filename causing this error was: %1.\nDo you want to create this file?").arg(fname), i18n("Cannot Find File"))
-			== KMessageBox::Yes)
-				{
-					emit(fileNew(url));
+			else{
+				QString otherFilename;
+
+				if( item->type() == KileStruct::Bibliography ){
+					otherFilename = m_ki->checkOtherPaths(fi.dirPath(),fi.fileName(),KileInfo::bibinputs);
+				}
+				else if( item->type() == KileStruct::Input ){
+					otherFilename = m_ki->checkOtherPaths(fi.dirPath(),fi.fileName(),KileInfo::texinputs);
+				}
+
+				fi.setFile(otherFilename);
+
+				if(fi.isReadable()){
+					url.setPath(otherFilename);
+					emit(fileOpen(url, QString::null));
+				}
+				else{
+					if ( KMessageBox::warningYesNo(this, i18n("Cannot find the included file. The file does not exist, is not readable or Kile is unable to determine the correct path to it. The filename causing this error was: %1.\nDo you want to create this file?").arg(fname), i18n("Cannot Find File"))
+				== KMessageBox::Yes) {
+						url.setPath(fname);
+						emit(fileNew(url));
+					}
 				}
 			}
 		}
 	}
 
+	// all popup items get different id's, so that we can see, what item is activated
+	//  - label:       1 -  6
+	//  - sectioning: 10 - 16
+	//  - graphics:   100ff
+
 	void Structure::slotPopup(KListView *, QListViewItem *itm, const QPoint &point)
 	{
-		kdDebug() << "\tStructure::slotPopup" << endl;
+		KILE_DEBUG() << "\tStructure::slotPopup" << endl;
 		
-		KileListViewItem *item = (KileListViewItem *)(itm);
-		if ( ! item )
+		m_popupItem = (KileListViewItem *)(itm);
+		if ( ! m_popupItem )
 			return;
 			
 		m_popup->clear();
 		m_popup->disconnect();
 		
-		m_popupItem = item;
-		
-		if ( ! item->label().isEmpty() )
+		bool hasLabel = ! m_popupItem->label().isEmpty();
+		if ( m_popupItem->type() == KileStruct::Sect )
 		{
-			m_popup->insertTitle(i18n("Insert Label"));
-			m_popup->insertItem(i18n("As &reference"),1);
-			m_popup->insertItem(i18n("As &page reference"),2);
-			m_popup->insertItem(i18n("Only the &label"),3);
+			if ( hasLabel )
+				m_popup->insertTitle(i18n("Sectioning"));
+			m_popup->insertItem(i18n("Cu&t"),SectioningCut);
+			m_popup->insertItem(i18n("&Copy"),SectioningCopy);
+			m_popup->insertItem(i18n("&Paste below"),SectioningPaste);
 			m_popup->insertSeparator();
-			m_popup->insertTitle(i18n("Copy Label to Clipboard"));
-			m_popup->insertItem(i18n("As reference"),4);
-			m_popup->insertItem(i18n("As page reference"),5);
-			m_popup->insertItem(i18n("Only the label"),6);
-			
-			connect(m_popup,  SIGNAL(activated(int)), this, SLOT(slotPopupLabel(int)));
-			m_popup->exec(point);
+			m_popup->insertItem(i18n("&Select"),SectioningSelect);
+			m_popup->insertItem(i18n("&Delete"),SectioningDelete);
+			m_popup->insertSeparator();
+			m_popup->insertItem(i18n("C&omment"),SectioningComment);
+			m_popup->insertSeparator();
+			m_popup->insertItem(i18n("Run QuickPreview"), SectioningPreview);
 		}
-		else if ( item->type() == KileStruct::Graphics )
+		else if ( m_popupItem->type() == KileStruct::Graphics )
 		{
-			m_popupInfo = item->title();
+			m_popupInfo = m_popupItem->title();
 						
 			if(m_popupInfo.left(1) != "/") // no absolute path
 			{
 				QString fn = m_ki->getCompileName();
-				m_popupInfo = QFileInfo(fn).dirPath() + "/" + m_popupInfo;
+				m_popupInfo = QFileInfo(fn).dirPath() + '/' + m_popupInfo;
 			}
 			
 			QFileInfo fi(m_popupInfo);
@@ -622,46 +774,81 @@ namespace KileWidget
 				
 				m_offerList = KTrader::self()->query(KMimeType::findByURL(url)->name(), "Type == 'Application'");
 				for (uint i=0; i < m_offerList.count(); ++i)
-					m_popup->insertItem(SmallIcon(m_offerList[i]->icon()), m_offerList[i]->name(), i+1);
+				{
+					m_popup->insertItem(SmallIcon(m_offerList[i]->icon()), m_offerList[i]->name(), i+SectioningGraphicsOfferlist);
+				}
 				m_popup->insertSeparator();
-				m_popup->insertItem(i18n("Other..."), 0);
-				
-				connect(m_popup, SIGNAL(activated(int)), this, SLOT(slotPopupRun(int)));
-				m_popup->exec(point);
+				m_popup->insertItem(i18n("Other..."), SectioningGraphicsOther);
 			}
 		}
+		
+		if ( hasLabel)
+		{
+			m_popup->insertTitle(i18n("Insert Label"));
+			m_popup->insertItem(i18n("As &reference"),1);
+			m_popup->insertItem(i18n("As &page reference"),2);
+			m_popup->insertItem(i18n("Only the &label"),3);
+			m_popup->insertSeparator();
+			m_popup->insertTitle(i18n("Copy Label to Clipboard"));
+			m_popup->insertItem(i18n("As reference"),4);
+			m_popup->insertItem(i18n("As page reference"),5);
+			m_popup->insertItem(i18n("Only the label"),6);
+		}
+
+		if ( m_popup->count() > 0 )
+		{
+			connect(m_popup,SIGNAL(activated(int)),this,SLOT(slotPopupActivated(int)));
+			m_popup->exec(point);
+		}
 	}
 
-	void Structure::slotPopupLabel(int id)
+	void Structure::slotPopupActivated(int id)
 	{
-		kdDebug() << "\tStructure::slotPopupLabel" << endl;
-		
 		if ( id>=1 && id<=6 )
-		{
-			QString s = m_popupItem->label();    
-			if ( id==1 || id==4 )
-				s = "\\ref{" + s + "}";
-			else if ( id==2 || id==5 )
-				s = "\\pageref{" + s + "}";
-				
-			if ( id <= 3 )
-				emit( sendText(s) );
-			else
-				QApplication::clipboard()->setText(s);
- 		}
+			slotPopupLabel(id);
+		else if ( id>=SectioningCut && id<=SectioningPreview )
+			slotPopupSectioning(id);
+		else if ( id>=SectioningGraphicsOther && id<=SectioningGraphicsOfferlist+(int)m_offerList.count() )
+			slotPopupGraphics(id);
 	}
 
-	void Structure::slotPopupRun(int id)
+	// id's 1..6 (already checked)
+	void Structure::slotPopupLabel(int id)
 	{
-		kdDebug() << "\tStructure::slotPopupRun" << endl;
+		KILE_DEBUG() << "\tStructure::slotPopupLabel (" << id << ")"<< endl;
 		
+		QString s = m_popupItem->label();    
+		if ( id==1 || id==4 )
+			s = "\\ref{" + s + '}';
+		else if ( id==2 || id==5 )
+			s = "\\pageref{" + s + '}';
+			
+		if ( id <= 3 )
+			emit( sendText(s) );
+		else
+			QApplication::clipboard()->setText(s);
+ 	}
+
+	// id's 10..16 (already checked)
+	void Structure::slotPopupSectioning(int id)
+	{
+		KILE_DEBUG() << "\tStructure::slotPopupSectioning (" << id << ")"<< endl;
+		if ( m_popupItem->level()>=1 && m_popupItem->level()<=7 )
+			emit( sectioningPopup(m_popupItem,id) );
+	}
+
+	// id's 100ff (already checked)
+	void Structure::slotPopupGraphics(int id)
+	{
+		KILE_DEBUG() << "\tStructure::slotPopupGraphics (" << id << ")"<< endl;
+
 		KURL url;
 		url.setPath(m_popupInfo);
 		
-		if (id == 0)
+		if ( id == SectioningGraphicsOther )
 			KRun::displayOpenWithDialog(url);
 		else
-			KRun::run(*m_offerList[id-1],url);
+			KRun::run(*m_offerList[id-SectioningGraphicsOfferlist],url);
 	}
 
 	StructureList* Structure::viewFor(KileDocument::Info *info)
@@ -707,9 +894,21 @@ namespace KileWidget
 		m_default->activate();
 	}
 
-	void Structure::update(KileDocument::Info *docinfo, bool parse)
+	void Structure::updateUrl(KileDocument::Info *docinfo)
 	{
-		kdDebug() << "==KileWidget::Structure::update(" << docinfo << ")=============" << endl;
+		StructureList *view = viewFor(docinfo);
+		if ( view )
+			view->updateRoot();
+	}
+
+	void Structure::update(KileDocument::Info *docinfo)
+	{
+		update(docinfo, true);
+	}
+
+	void Structure::update(KileDocument::Info *docinfo, bool parse, bool activate /* =true */)
+	{
+		KILE_DEBUG() << "==KileWidget::Structure::update(" << docinfo << ")=============" << endl;
 
 		if ( docinfo == 0L ) 
 		{
@@ -726,26 +925,32 @@ namespace KileWidget
 
 		if ( needParsing ) //need to reparse the doc
 		{
-			//kdDebug() << "\tStructure::update parsing doc" << endl;
+			int xtop = view->contentsX();
+			int ytop = view->contentsY();
+			//KILE_DEBUG() << "\tStructure::update parsing doc" << endl;
 			view->cleanUp();
 			m_docinfo->updateStruct();
 			view->showReferences(m_ki);
+			view->setContentsPos(xtop,ytop);
 		}
 
-		kdDebug() << "\tStructure::update activating view" << endl;
-		view->activate();
+		if(activate)
+		{
+			KILE_DEBUG() << "===Structure::update() activating view" << endl;
+			view->activate();
+		}
 	}
 
     void Structure::clean(KileDocument::Info *docinfo)
     {
-        kdDebug() << "==void Structure::clean()========" << endl;
+        KILE_DEBUG() << "==void Structure::clean()========" << endl;
         StructureList *view = viewFor(docinfo);
         if (view) view->cleanUp();
     }
 
 	void Structure::updateReferences(KileDocument::Info *docinfo)
 	{
-		kdDebug() << "==void StructureList::updateReferences()========" << endl;
+		KILE_DEBUG() << "==void StructureList::updateReferences()========" << endl;
 		StructureList *view = viewFor(docinfo);
 		if (view) 
 		{
@@ -753,6 +958,58 @@ namespace KileWidget
 		}
 	}
 
+	////////////////////// Structure: find sectioning //////////////////////
+
+	bool Structure::findSectioning(KileListViewItem *refItem, Kate::Document *doc, uint row, uint col, bool backwards, bool checkLevel, uint &sectRow, uint &sectCol)
+	{
+		KileDocument::TextInfo *docinfo = m_ki->docManager()->textInfoFor(doc);
+		
+		if ( ! docinfo ){
+			return false;
+		}
+		
+		if( checkLevel && !refItem ){ // only allow a refItem == NULL if checkLevel is false
+			return false;
+		}
+		
+		bool found = false;
+		uint foundRow,foundCol;
+		StructureList *structurelist = viewFor(docinfo);
+		QListViewItemIterator it( structurelist );
+		while ( it.current() ) 
+		{
+			KileListViewItem *item = (KileListViewItem *)(it.current());
+			if  (item->type() == KileStruct::Sect && (!checkLevel || item->level() <= refItem->level()))
+			{
+				foundRow = item->startline() - 1;
+				foundCol = item->startcol() - 1;
+				if ( backwards )
+				{
+					if ( foundRow<row || (foundRow==row &&foundCol<col) )
+					{
+						sectRow = foundRow;
+						sectCol = foundCol;
+						found = true;
+					}
+					else
+					{
+						return found;
+					}
+					
+				}
+				else if ( !backwards && (foundRow>row || (foundRow==row &&foundCol>col)) )
+				{
+					sectRow = foundRow;
+					sectCol = foundCol;
+					return true;
+				}
+			}
+			++it;
+		}
+
+		return found;
+	}
+
 }
 
 #include "kilestructurewidget.moc"
